function math.sign(n) return n>0 and 1 or n<0 and -1 or 0 end

function love.load()

    wf = require'Libraries/windfield'
    world = wf.newWorld(0, 0)

    Camera = require'Libraries.camera'.new()
    -- initalize planets
    planets = {
        {
            ['name'] = 'earth',
            ['mass'] = 6.5*10^24,
            ['radius'] = 63700/2,
            ['atmosphere'] = {
                ['surfacepressure'] = 101.325, -- kPa
                ['atmospherecolour'] = nil,
                ['hasAtmosphere'] = true,
                ['color'] = {
                    R = 135/255,
                    G = 206/255,
                    B = 235/255,
                }
            },
            ['x'] = 63700,
            ['y'] = 0,
            ['color'] = {
                R = 0/255,
                G = 154/255,
                B = 23/255,
            },
            collider = nil -- do not change
        },
        {
            ['name'] = 'moon',
            ['mass'] = 7.342*10^23,
            ['radius'] = 17300/2,
            ['atmosphere'] = {
                ['surfacepressure'] = 101.325, -- kPa
                ['atmospherecolour'] = nil,
                ['hasAtmosphere'] = false,
                ['color'] = {
                    R = 135/255,
                    G = 206/255,
                    B = 235/255,
                }
            },
            ['x'] = -(17300)*5,
            ['y'] = 0,
            ['color'] = {
                R = 184/255,
                G = 174/255,
                B = 163/255,
            },
            collider = nil -- do not change
        },
    }

    -- player setup
    Player = {}
    Player.collider = world:newCircleCollider(400, 250, 64)
    Player.collider:setFixedRotation(true)
    Player.collider:setType('dynamic')
    Player.collider:setFriction(0.1)
    Player.Position = {
        X = 0,
        Y = 0,
    }
    Player.Velocity = {
        X = -(17300)*5,
        Y = 0
    }

    G = 6.6743e-11 -- gravitational constant
    -- setup planets
    for index, Planet in pairs(planets) do
        shape = world:newCircleCollider(Planet.x, Planet.y, Planet.radius*2)
        shape:setType('static')
        Planet.collider = shape
        Planet.atmosphere.atmosphere = love.graphics.newImage('Images/atmosphere_fade.png')
        Planet.orbitalVelocity = math.sqrt(G * Planet.mass / (6.451 * 10^6))
    end
    atmospherePressure = 0
    -- shaders
    atmosphereShader = love.graphics.newShader([[
        extern Image texture;
        extern float baseFadeDistance;
        extern vec2 planetCenter;
    
        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
            // Get the current pixel color
            vec4 pixel = Texel(texture, texture_coords);
    
            vec2 diff = screen_coords - planetCenter;
            float distance = length(diff);
    
            float fadeDistance = baseFadeDistance * (1 - distance / baseFadeDistance);
    
            float dy = abs(screen_coords.y - planetCenter.y);
    
            float alpha = 1.0 - exp(distance/fadeDistance);
            alpha = alpha;
            alpha = clamp(alpha, 0.0, 1.0);
    
            // Output the modified pixel color with the new alpha value
            return vec4(pixel.rgb, alpha * pixel.a);
        }
    ]])
    -- Set the values for the uniforms in the atmosphereShader
    local screenWidth, screenHeight = love.graphics.getDimensions()
    atmosphereShader:send("baseFadeDistance", 6370/2)
    atmosphereShader:send("planetCenter", {6370/screenWidth, 6370/screenHeight})

end

function love.update(dt)
    -- update shaders
    local distanceToPlanet = math.sqrt((Camera.x + (6370))^2 + (Camera.y + (6370))^2)
    atmosphereShader:send("baseFadeDistance", distanceToPlanet * 0.5)
    -- stuff
    world:update(dt)
    local vX = 0
    local vY = 0

    if love.keyboard.isDown('w') then
        vY = vY - math.cos(Player.collider:getAngle()) * 1000 * dt
    end 
    if love.keyboard.isDown('s') then
        vY = vY - math.cos(Player.collider:getAngle()) * 1000 * -dt
    end

    if love.keyboard.isDown('a') then
        vX = vX - math.cos(Player.collider:getAngle()) * 1000 * dt
    end
    if love.keyboard.isDown('d') then
        vX = vX - math.cos(Player.collider:getAngle()) * 1000 * -dt
    end
    Player.collider:applyForce(vX*1e4, vY*1e4)

    -- planet stuff
    for index, Planet in pairs(planets) do
        -- gravity
        local dx = Planet.x - Player.Position.X
        local dy = Planet.y - Player.Position.Y
        local r = math.sqrt(dx^2 + dy^2)

        local m1 = 1 -- player mass, 100kg
        local m2 = Planet.mass
        local F = G * m1 * m2 / r^2
        local a = F/m1

        local angle = math.atan2(dy, dx)
        local aX = math.cos(angle) * a
        local aY = math.sin(angle) * a

        Player.collider:applyForce(aX, aY)
        -- make player camera rotate to be perpendicular with surface
        local cameraDistance = r + Planet.radius
        local distanceRatio = Planet.radius/cameraDistance
        if distanceRatio > 0.15 then
            local dx = Planet.x - Player.Position.X
            local dz = Planet.y - Player.Position.Y
            local distance = math.sqrt(dx^2 + dz^2)
            local dy = distance - cameraDistance
            local perpAngle = math.atan2(dx, dz)

            Camera:rotateTo(perpAngle, 0)
        end
        -- simulate atmospheric pressure
        if (Planet.atmosphere.hasAtmosphere and distanceRatio > 0.15) then
            local dx = (Planet.x - Player.Position.X)
            local dy = (Planet.y - Player.Position.Y)

            local surfacePressure = Planet.atmosphere.surfacepressure

            g = (G * Planet.mass / (Planet.radius*100)^2)
            local scaleHeight = (28.96 * 250)/(0.029*g)

            local altitude = (math.sqrt(dx^2 + dy^2) - (Planet.radius*2))/2
            local pressureAtAltitude = surfacePressure * math.exp(-altitude / scaleHeight)
        
            -- calculate pressure gradient
            local pressureGradient = (surfacePressure / scaleHeight) * math.exp(-altitude / scaleHeight*20)
            
            -- calculate atmospheric pressure at player's position
            atmospherePressure = pressureAtAltitude * math.exp(-altitude / scaleHeight)
            
            local rho = atmospherePressure / (287 * 250)

            local pVX, pVY = Player.collider:getLinearVelocity()

            local dragFX = .8 * 70 * 0.5 * rho * pVX^2 * math.sign(pVX)
            local dragFY = .8 * 70 * 0.5 * rho * pVY^2 * math.sign(pVY)
            Player.collider:applyForce(-dragFX, -dragFY)
        end
    end
    -- apply total velocity

    local pX, pY = Player.collider:getLinearVelocity()

    Player.Position = {
        X = Player.collider:getX(),
        Y = Player.collider:getY(),
    }
    local dx,dy = Player.Position.X, Player.Position.Y
    Camera:lockPosition(dx, dy)
    Player.Velocity.X = pX
    Player.Velocity.Y = pY
end

function love.draw()
    Camera:attach()    
    -- draw planet bodies
    for index, Planet in pairs(planets) do
        local pX, pY = Planet.x, Planet.y
        -- planet is last, so front layer.
        if Planet.atmosphere.hasAtmosphere then
            local width = Planet.atmosphere.atmosphere:getWidth()
            local height = Planet.atmosphere.atmosphere:getHeight()
            
            love.graphics.setShader(atmosphereShader)
            --love.graphics.draw(Planet.atmosphere.atmosphere, pX, pY, 0, (Planet.radius/(width/2))*2.0002, (Planet.radius/(height/2))*2.0002, width/2, height/2)
            love.graphics.circle('fill', pX, pY, Planet.radius*2.05)
            love.graphics.setShader()
            love.graphics.setColor(1,1,1)
        end
        love.graphics.setColor(Planet.color.R, Planet.color.G, Planet.color.B)
        love.graphics.circle('fill', pX, pY, Planet.radius*2)
        love.graphics.setColor(1,1,1)
    end
    world:draw()
    Camera:detach()
    love.graphics.print(Player.Position.X, 0, 20)
    love.graphics.print(Player.Position.Y, 0, 40)
    love.graphics.print(atmospherePressure .. ' kPa', 0, 60)
    love.graphics.print(g.. ' m/s', 0, 80)

    dx = planets[1].x - Player.Position.X
    dy = planets[1].y - Player.Position.Y
    r = math.sqrt(dx^2 + dy^2)
end